/*
 * Copyright (c) 2012 David MASSENOT - http://artcustomer.fr/
 * 
 * Permission is hereby granted to use, modify, and distribute this file 
 * in accordance with the terms of the license agreement accompanying it.
 */

package artcustomer.framework.extensions.alternativa3d.controller {
	import flash.display.InteractiveObject;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.events.KeyboardEvent;
	
	import alternativa.engine3d.controllers.SimpleObjectController;
	import alternativa.engine3d.core.Object3D;
	
	
	/**
	 * RollCameraController : Alternativa3D 8.32
	 * 
	 * @author David Massenot
	 */
	public class RollCameraController extends SimpleObjectController {
		public static const ACTION_ZOOM_IN:String = 'actionZoomIn';
		public static const ACTION_ZOOM_OUT:String = 'actionZoomOut';
		
		private var _eventSource:InteractiveObject;
		private var _target:Object3D;
		
		public var minZ:Number = 100;
		public var maxZ:Number = 800;
		public var minLength:Number = 200;
		public var maxLength:Number = 2000;
		
		private var _easingSeparator:Number = 20;
		private var _yawNear:Boolean = false;
		private var _yawNearAngle:Number = 0;
		private var _pitchNear:Boolean = false;
		private var _pitchNearAngle:Number = 0;
		
		private var _pitchDown:Boolean = false;
		private var _pitchUp:Boolean = false;
		private var _yawLeft:Boolean = false;
		private var _yawRight:Boolean = false;
		
		private var _mouseMove:Boolean;
		private var _mouseSensitivityX:Number = 1;
		private var _mouseSensitivityY:Number = 1;
		private var _mouseX:Number = 0;
		private var _mouseY:Number = 0;
		private var _zoomIn:Boolean = false;
		private var _zoomOut:Boolean = false;
		private var _length:Number = 700;
		private var _lastLength:Number = 700;
		
		private var _zoomSpeed:Number = 10;
		private var _yawSpeed:Number = 5;
		private var _pitchSpeed:Number = 30;
		
		private var _oldAngle:Number;
		private var _angle:Number = 90;
		private var _lastAngle:Number = _angle;
		private var _oldZ:Number = 0;
		private var _lastZ:Number = 0;
		
		private var _lookAtX:Number = 0;;
		private var _lastLookAtX:Number = _lookAtX;
		private var _lookAtY:Number = 0;
		private var _lastLookAtY:Number = _lookAtY;
		private var _lookAtZ:Number = 0;
		private var _lastLookAtZ:Number = _lookAtZ;
		
		
		/**
		 * Constructor
		 * 
		 * @param	eventSource
		 * @param	target
		 * @param	speed
		 * @param	speedMultiplier
		 * @param	mouseSensitivityX
		 * @param	mouseSensitivityY
		 */
		public function RollCameraController(eventSource:InteractiveObject, target:Object3D, speed:Number, speedMultiplier:Number = 3, mouseSensitivityX:Number = 1, mouseSensitivityY:Number = 1) {
			_eventSource = eventSource;
			_target = target;
			
			_lastZ = _target.z;
			
			super(eventSource, target, speed, speedMultiplier, mouseSensitivity);
			listenEvents();
		}
		
		//---------------------------------------------------------------------
		//  Events
		//---------------------------------------------------------------------
		
		/**
		 * @private
		 */
		private function listenEvents():void {
			_eventSource.addEventListener(KeyboardEvent.KEY_DOWN, handleKeyboard, false, 0, true);
			_eventSource.addEventListener(KeyboardEvent.KEY_UP, handleKeyboard, false, 0, true);
			_eventSource.addEventListener(MouseEvent.MOUSE_WHEEL, handleMouse, false, 0, true);
		}
		
		/**
		 * @private
		 */
		private function unlistenEvents():void {
			_eventSource.removeEventListener(KeyboardEvent.KEY_DOWN, handleKeyboard);
			_eventSource.removeEventListener(KeyboardEvent.KEY_UP, handleKeyboard);
			_eventSource.removeEventListener(MouseEvent.MOUSE_WHEEL, handleMouse);
		}
		
		//---------------------------------------------------------------------
		//  Listeners
		//---------------------------------------------------------------------
		
		/**
		 * @private
		 */
		private function handleMouse(e:MouseEvent):void {
			switch (e.type) {
				case('mouseWheel'):
					_lastLength += e.delta * 20;
					
					if (_lastLength < minLength) {
						_lastLength = minLength;
					} else if (_lastLength > maxLength) {
						_lastLength = maxLength;
					}
					break;
			}
		}
		
		/**
		 * @private
		 */
		private function handleKeyboard(e:KeyboardEvent):void {
			var key:String;
			
			switch (e.type) {
				case('keyDown'):
					for (key in keyBindings) {
						if (String(e.keyCode) == key) {
							keyBindings[key](true);
						}
					}
					break;
					
				case('keyUp'):
					for (key in keyBindings) {
						keyBindings[key](false);
					}
					
					_yawNear = false;
					_pitchNear = false;
					break;
			}
		}
		
		
		/**
		 * Bind a key.
		 * 
		 * @param	keyCode
		 * @param	action
		 */
		override public function bindKey(keyCode:uint, action:String): void {
			switch (action) {
				case ACTION_ZOOM_IN:
					keyBindings[keyCode] = zoomIn;
					break;
					
				case ACTION_ZOOM_OUT:
					keyBindings[keyCode] = zoomOut;
					break;
					
				case ACTION_YAW_LEFT:
					keyBindings[keyCode] = yawLeft;
					break;
					
				case ACTION_YAW_RIGHT:
					keyBindings[keyCode] = yawRight;
					break;
					
				case ACTION_PITCH_DOWN:
					keyBindings[keyCode] = pitchDown;
					break;
					
				case ACTION_PITCH_UP:
					keyBindings[keyCode] = pitchUp;
					break;
			}
		}
		
		/**
		 * Update controller.
		 */
		override public function update(): void {
			var RADIAN:Number = Math.PI / 180;
			
			if (_zoomIn) {
				_lastLength -= _zoomSpeed;
			} else if (_zoomOut){
				_lastLength += _zoomSpeed;
			}
			
			if (_lastLength < minLength) {
				_lastLength = minLength;
			} else if (_lastLength > maxLength) {
				_lastLength = maxLength;
			}
			
			if (_lastLength - _length) _length += (_lastLength - _length) / _easingSeparator;
			
			if (_mouseMove) {
				_lastAngle = _oldAngle + (_eventSource.mouseX - _mouseX) * _mouseSensitivityX / 2;
			} else if (_yawLeft) {
				_lastAngle += _yawSpeed;
				
			} else if (_yawRight) {
				_lastAngle -= _yawSpeed;
			}
			
			if (_lastAngle-_angle) _angle += (_lastAngle - _angle) / _easingSeparator;
			
			_target.x = Math.cos(_angle * RADIAN) * _length;
			_target.y = Math.sin(_angle * RADIAN) * _length;
			
			if (_mouseMove) {
				_lastZ = _oldZ + (_eventSource.mouseY - _mouseY) * _mouseSensitivityY * 2;
			} else if (_pitchDown) {
				_lastZ -= _pitchSpeed;
			} else if (_pitchUp) {
				_lastZ += _pitchSpeed;
			}
			
			if (_lastZ < minZ) {
				_lastZ = minZ;
			} else if (_lastZ > maxZ) {
				_lastZ = maxZ;
			}
			
			if (_lastZ - _target.z) _target.z += (_lastZ - _target.z) / _easingSeparator;
			if (_lastLookAtX - _lookAtX) _lookAtX += (_lastLookAtX-_lookAtX) / _easingSeparator;
			if (_lastLookAtY - _lookAtY) _lookAtY += (_lastLookAtY-_lookAtY) / _easingSeparator;
			if (_lastLookAtZ - _lookAtZ) _lookAtZ += (_lastLookAtZ-_lookAtZ) / _easingSeparator;
			
			updateObjectTransform()
			lookAtXYZ(_lookAtX, _lookAtY, _lookAtZ);
		}
		
		/**
		 * Start mouse looking.
		 */
		override public function startMouseLook():void {
			_oldAngle = _lastAngle
			_oldZ = _lastZ
			_mouseX = _eventSource.mouseX;
			_mouseY = _eventSource.mouseY;
			_mouseMove = true;
			
		}
		
		/**
		 * Stop mouse looking.
		 */
		override public function stopMouseLook():void {
			_mouseMove = false;
		}
		
		
		/**
		 * Pitch down.
		 * 
		 * @param	value
		 */
		public function pitchDown(value:Boolean):void {
			_pitchDown = value;
		}
		
		/**
		 * Pitch up.
		 * 
		 * @param	value
		 */
		public function pitchUp(value:Boolean):void {
			_pitchUp = value;
		}
		
		/**
		 * Yaw on left.
		 * 
		 * @param	value
		 * @param	near
		 */
		public function yawLeft(value:Boolean, near:Boolean = false):void {
			_yawLeft = value;
			_yawNear = near;
		}
		
		/**
		 * Yaw on right.
		 * 
		 * @param	value
		 * @param	near
		 */
		public function yawRight(value:Boolean, near:Boolean = false):void {
			_yawRight = value;
			_yawNear = near;
		}
		
		/**
		 * Zoom in.
		 * 
		 * @param	value
		 */
		public function zoomIn(value:Boolean):void {
			_zoomIn = value;
		}
		
		/**
		 * Zoom out.
		 * 
		 * @param	value
		 */
		public function zoomOut(value:Boolean):void {
			_zoomOut = value;
		}
		
		/**
		 * Look at a specific position.
		 * 
		 * @param	x
		 * @param	y
		 * @param	z
		 * @param	mode
		 */
		public function lookAtPosition(x:Number, y:Number, z:Number, mode:Boolean = false):void {
			if (mode) {
				_lookAtX = x;
				_lookAtY = y;
				_lookAtZ = z;
			}
			
			_lastLookAtX = x;
			_lastLookAtY = y;
			_lastLookAtZ = z;
		}
		
		/**
		 * Define angle.
		 * 
		 * @param	value
		 * @param	mode
		 */
		public function angle(value:Number, mode:Boolean = false):void {
			if (mode) {
				_angle = value + 90;
			}
			
			_lastAngle = value + 90;
		}
		
		/**
		 * Define length
		 * 
		 * @param	value
		 * @param	mode
		 */
		public function length(value:Number, mode:Boolean = false):void {
			if (mode) {
				_length = value;
			}
			
			_lastLength = value;
		}
		
		/**
		 * Define posZ
		 * 
		 * @param	value
		 * @param	mode
		 */
		public function posZ(value:Number, mode:Boolean = false):void {
			if (mode) {
				_target.z = value;
			}
			
			_lastZ = value;
		}
		
		/**
		 * Destroy controller.
		 */
		public function destroy():void {
			unlistenEvents();
		}
		
		
		/**
		 * @private
		 */
		public function set zoomSpeed(value:Number):void {
			_zoomSpeed = value;
		}
		
		/**
		 * @private
		 */
		public function set yawSpeed(value:Number):void {
			_yawSpeed = value * Math.PI / 180;
		}
		
		/**
		 * @private
		 */
		public function set pitchSpeed(value:Number):void {
			_pitchSpeed = value
		}
		
		/**
		 * @private
		 */
		public function set mouseSensitivityX(value:Number):void {
			_mouseSensitivityX = value;
		}
		
		/**
		 * @private
		 */
		public function set mouseSensitivityY(value:Number):void {
			_mouseSensitivityY = value;
		}
		
		/**
		 * @private
		 */
		public function set easingSeparator(value:uint):void {
			if (value) {
				_easingSeparator = value;
			} else {
				_easingSeparator = 1;
			}
		}
	}
}