/*
 * Copyright (c) 2012 David MASSENOT - http://artcustomer.fr/
 * 
 * Permission is hereby granted to use, modify, and distribute this file 
 * in accordance with the terms of the license agreement accompanying it.
 */

package artcustomer.framework.extensions.alternativa3d.template {
	import flash.geom.Vector3D;
	import flash.ui.Keyboard;
	
	import alternativa.engine3d.core.Camera3D;
	import alternativa.engine3d.core.Object3D;
	import alternativa.engine3d.core.Resource;
	import alternativa.engine3d.core.View;
	import alternativa.engine3d.controllers.SimpleObjectController;
	
	import artcustomer.framework.engines.layers.tools.template.AbstractEngineTemplate;
	import artcustomer.framework.extensions.alternativa3d.controller.RollCameraController;
	
	
	/**
	 * AlternativaTemplate 8.32
	 * 
	 * @author David Massenot
	 */
	public class AlternativaTemplate extends AbstractEngineTemplate {
		private var _scene:Object3D;
		private var _camera:Camera3D;
		private var _view:View;
		
		private var _controller:SimpleObjectController;
		private var _rollController:RollCameraController;
		
		private var _useController:Boolean;
		private var _useRollController:Boolean;
		
		
		/**
		 * Constructor
		 */
		public function AlternativaTemplate() {
			super();
		}
		
		//---------------------------------------------------------------------
		//  3D Engine - Scene
		//---------------------------------------------------------------------
		
		/**
		 * @private
		 */
		private function setupScene():void {
			_scene = new Object3D();
		}
		
		/**
		 * @private
		 */
		private function destroyScene():void {
			if (_scene) _scene = null;
		}
		
		//---------------------------------------------------------------------
		//  3D Engine - View
		//---------------------------------------------------------------------
		
		/**
		 * @private
		 */
		private function setupView():void {
			var logoMargin:int = 100;
			
			_view = new View(this.layer3D.context.contextWidth, this.layer3D.context.contextHeight, false, this.layer3D.stage.color, 1, 4);
			_view.contextMenu = null;
			_view.logoHorizontalMargin = logoMargin;
		}
		
		/**
		 * @private
		 */
		private function destroyView():void {
			if (_view) _view = null;
		}
		
		//---------------------------------------------------------------------
		//  3D Engine - Camera
		//---------------------------------------------------------------------
		
		/**
		 * @private
		 */
		private function setupCamera():void {
			_camera = new Camera3D(1, 100000);
			_camera.view = _view;
			
			this.layer3D.viewPort.addChild(_camera.view);
			
			_scene.addChild(_camera);
		}
		
		/**
		 * @private
		 */
		private function renderCamera():void {
			if (_camera) {
				_camera.view.width = this.layer3D.context.contextWidth;
				_camera.view.height = this.layer3D.context.contextHeight;
				
				_camera.render(this.layer3D.stage3D);
			}
		}
		
		/**
		 * @private
		 */
		private function destroyCamera():void {
			if (_camera) {
				_camera.view = null;
				_camera = null;
			}
		}
		
		//---------------------------------------------------------------------
		//  3D Engine - Controller
		//---------------------------------------------------------------------
		
		/**
		 * @private
		 */
		private function setupController():void {
			_controller = new SimpleObjectController(this.layer3D.stage, _camera, 200);
			_controller.lookAt(new Vector3D(0, 0, 0));
		}
		
		/**
		 * @private
		 */
		private function updateController():void {
			if (_controller) _controller.update();
		}
		
		/**
		 * @private
		 */
		private function destroyController():void {
			if (_controller) {
				_controller = null;
			}
		}
		
		//---------------------------------------------------------------------
		//  3D Engine - RollController
		//---------------------------------------------------------------------
		
		/**
		 * @private
		 */
		private function setupRollController():void {
			_rollController = new RollCameraController(this.layer3D.stage, _camera, 200);
			_rollController.mouseSensitivity = 0;
			_rollController.mouseSensitivityX = .6; 
			_rollController.mouseSensitivityY = .6; 
			_rollController.easingSeparator = 30;
			_rollController.minLength = 200;
			_rollController.minZ = -1000;
			_rollController.maxZ = 1000;
			
			_rollController.unbindAll();
			_rollController.accelerate(true);
			_rollController.length(70);
			_rollController.posZ(500, true);
			
			_rollController.angle(0);
			_rollController.lookAtPosition(0, 0, 10);
			
			_rollController.bindKey(Keyboard.LEFT, SimpleObjectController.ACTION_YAW_LEFT);
			_rollController.bindKey(Keyboard.RIGHT,SimpleObjectController.ACTION_YAW_RIGHT);
			_rollController.bindKey(Keyboard.DOWN, SimpleObjectController.ACTION_PITCH_DOWN);
			_rollController.bindKey(Keyboard.UP, SimpleObjectController.ACTION_PITCH_UP);
			_rollController.bindKey(Keyboard.PAGE_UP, RollCameraController.ACTION_ZOOM_IN);
			_rollController.bindKey(Keyboard.PAGE_DOWN, RollCameraController.ACTION_ZOOM_OUT);
		}
		
		/**
		 * @private
		 */
		private function updateRollController():void {
			if (_rollController) _rollController.update();
		}
		
		/**
		 * @private
		 */
		private function destroyRollController():void {
			if (_rollController) {
				_rollController.destroy();
				_rollController = null;
			}
		}
		
		
		/**
		 * Upload scene resources.
		 */
		public function uploadSceneResources():void {
			var resources:Vector.<Resource> = _scene.getResources(true);
			var resource:Resource;
			var i:int;
			var length:int = resources.length;
			
			for (i ; i < length ; ++i) {
				resource = resources[i];
				resource.upload(this.layer3D.stage3D.context3D);
			}
		}
		
		/**
		 * Upload resources.
		 * 
		 * @param	resources
		 */
		public function uploadResources(resources:Vector.<Resource>):void {
			if (!resources) return;
			
			var resource:Resource;
			var i:int;
			var length:int = resources.length;
			
			for (i ; i < length ; ++i) {
				resource = resources[i];
				resource.upload(this.layer3D.stage3D.context3D);
			}
		}
		
		/**
		 * Move camera.
		 * 
		 * @param	x
		 * @param	y
		 * @param	z
		 * @param	rotationX
		 * @param	rotationY
		 * @param	rotationZ
		 */
		public function moveCamera(x:Number = 0, y:Number = 0, z:Number = 0, rotationX:Number = 0, rotationY:Number = 0, rotationZ:Number = 0):void {
			if (_camera) {
				_camera.x = x;
				_camera.y = y;
				_camera.z = z;
				_camera.rotationX = rotationX;
				_camera.rotationY = rotationY;
				_camera.rotationZ = rotationZ;
			}
		}
		
		/**
		 * Setup template.
		 */
		override public function setup():void {
			super.setup();
			
			_useController = true;
			_useRollController = true;
			
			if (!_useController) _useRollController = false;
			
			setupScene();
			setupView();
			setupCamera();
			
			if (_useRollController) {
				setupRollController();
			} else if (_useController) {
				setupController();
			}
		}
		
		/**
		 * Destroy instance.
		 */
		override public function destroy():void {
			destroyView();
			destroyCamera();
			destroyScene();
			destroyController();
			destroyRollController();
			
			_useController = false;
			_useRollController = false;
			
			super.destroy();
		}
		
		/**
		 * Render template.
		 */
		override public function render():void {
			updateController();
			updateRollController();
			renderCamera();
		}
		
		
		/**
		 * @private
		 */
		public function get scene():Object3D {
			return _scene;
		}
		
		/**
		 * @private
		 */
		public function get camera():Camera3D {
			return _camera;
		}
		
		/**
		 * @private
		 */
		public function get view():View {
			return _view;
		}
		
		/**
		 * @private
		 */
		public function get controller():SimpleObjectController {
			return _controller;
		}
		
		/**
		 * @private
		 */
		public function get rollController():RollCameraController {
			return _rollController;
		}
	}
}