/*
 * Copyright (c) 2013 David MASSENOT - http://artcustomer.fr/
 * 
 * Permission is hereby granted to use, modify, and distribute this file 
 * in accordance with the terms of the license agreement accompanying it.
 */

package artcustomer.framework.extensions.display.ui.list.item {
	import flash.display.Sprite;
	import flash.utils.getQualifiedClassName;
	
	import artcustomer.framework.base.IDestroyable;
	import artcustomer.framework.errors.IllegalError;
	import artcustomer.framework.utils.tools.StringTools;
	import artcustomer.framework.extensions.display.ui.list.events.MenuListEvent;
	import artcustomer.framework.extensions.display.ui.list.data.AbstractListItemValueObject;
	
	
	/**
	 * AbstractListItem
	 * 
	 * @author David Massenot
	 */
	public class AbstractListItem extends Sprite implements IDestroyable {
		public static const DEFAULT_STATE:String = 'defaultState';
		
		private static const FULL_CLASS_NAME:String = 'artcustomer.maxima.extensions.display.ui.list.item::AbstractListItem';
		
		private var _valueObject:AbstractListItemValueObject;
		
		private var _state:String;
		
		private var _isLocked:Boolean;
		
		
		/**
		 * Constructor
		 */
		public function AbstractListItem() {
			_state = DEFAULT_STATE;
			
			if (getQualifiedClassName(this) == FULL_CLASS_NAME) throw new IllegalError(IllegalError.E_ABSTRACT_CLASS);
		}
		
		//---------------------------------------------------------------------
		//  Event Dispatching
		//---------------------------------------------------------------------
		
		/**
		 * @private
		 */
		private function dispatchMenuListEvent(type:String):void {
			this.dispatchEvent(new MenuListEvent(type, true, false, _valueObject, _state));
		}
		
		
		/**
		 * Called when current state change. Override it to use states !
		 */
		protected function onStateChange():void {
			
		}
		
		/**
		 * Called when value object is updated. Override it !
		 */
		protected function onValueChange():void {
			
		}
		
		
		/**
		 * Entry point. Override it !
		 */
		public function setup():void {
			
		}
		
		/**
		 * Destructor. Override it and call it at end !
		 */
		public function destroy():void {
			_valueObject = null;
			_state = null;
			_isLocked = false;
		}
		
		/**
		 * Update. Override it !
		 */
		public function update():void {
			
		}
		
		/**
		 * Select item. Override it and call it at end !
		 */
		public function select():void {
			dispatchMenuListEvent(MenuListEvent.ON_SELECT_ITEM);
		}
		
		/**
		 * Focus item. Override it and call it at end !
		 */
		public function focus():void {
			dispatchMenuListEvent(MenuListEvent.ON_FOCUS_ITEM);
		}
		
		/**
		 * Unfocus item. Override it and call it at end !
		 */
		public function unfocus():void {
			dispatchMenuListEvent(MenuListEvent.ON_UNFOCUS_ITEM);
		}
		
		/**
		 * Lock item. Override it and call it at end !
		 */
		public function lock():void {
			if (!_isLocked) {
				_isLocked = true;
				
				dispatchMenuListEvent(MenuListEvent.ON_LOCK_ITEM);
			}
		}
		
		/**
		 * Unlock item. Override it and call it at end !
		 */
		public function unlock():void {
			if (_isLocked) {
				_isLocked = false;
				
				dispatchMenuListEvent(MenuListEvent.ON_UNLOCK_ITEM);
			}
		}
		
		/**
		 * Set new state
		 * 
		 * @param	state
		 */
		public function setState(state:String):void {
			if (state != _state) {
				_state = state;
				
				this.onStateChange();
			}
		}
		
		
		/**
		 * Get String format of object.
		 * 
		 * @return
		 */
		override public function toString():String {
			return StringTools.formatToString(this, 'AbstractListItem', 'valueObject');
		}
		
		
		/**
		 * @private
		 */
		public function get valueObject():AbstractListItemValueObject {
			return _valueObject;
		}
		
		/**
		 * @private
		 */
		public function set valueObject(value:AbstractListItemValueObject):void {
			if (value != _valueObject) {
				_valueObject = value;
				
				onValueChange();
			}
		}
		
		/**
		 * @private
		 */
		public function get state():String {
			return _state;
		}
		
		/**
		 * @private
		 */
		public function get isLocked():Boolean {
			return _isLocked;
		}
	}
}