/*
 * Copyright (c) 2012 David MASSENOT - http://artcustomer.fr/
 * 
 * Permission is hereby granted to use, modify, and distribute this file 
 * in accordance with the terms of the license agreement accompanying it.
 */

package artcustomer.framework.extensions.minko.template {
	import artcustomer.framework.engines.layers.tools.template.AbstractEngineTemplate;
	
	import aerys.minko.render.Viewport;
	import aerys.minko.scene.node.Scene;
	import aerys.minko.scene.node.camera.Camera;
	import aerys.minko.scene.controller.camera.ArcBallController;
	
	
	/**
	 * MinkoTemplate 2.1
	 * 
	 * @author David Massenot
	 */
	public class MinkoTemplate extends AbstractEngineTemplate {
		private var _viewport:Viewport;
		private var _camera:Camera;
		private var _scene:Scene;
		private var _controller:ArcBallController;
		
		
		/**
		 * Constructor
		 */
		public function MinkoTemplate() {
			super();
		}
		
		//---------------------------------------------------------------------
		//  Viewport
		//---------------------------------------------------------------------
		
		/**
		 * @private
		 */
		private function setupViewport():void {
			_viewport = new Viewport();
			_viewport.backgroundColor = 0x000000;
			
			this.layer3D.stage.addChild(_viewport);
		}
		
		/**
		 * @private
		 */
		private function destroyViewport():void {
			if (_viewport) {
				this.layer3D.stage.removeChild(_viewport);
				
				_viewport.dispose();
				_viewport = null;
			}
		}
		
		//---------------------------------------------------------------------
		//  Scene
		//---------------------------------------------------------------------
		
		/**
		 * @private
		 */
		private function setupScene():void {
			_scene = new Scene();
		}
		
		/**
		 * @private
		 */
		private function destroyScene():void {
			if (_scene) _scene = null;
		}
		
		//---------------------------------------------------------------------
		//  Controller
		//---------------------------------------------------------------------
		
		/**
		 * @private
		 */
		private function setupController():void {
			_controller = new ArcBallController();
			_controller.bindDefaultControls(this.layer3D.stage);
			_controller.minDistance = 1;
			_controller.yaw = Math.PI * -.5;
			_controller.pitch = Math.PI / 2;
			_controller.distance = 5;
		}
		
		/**
		 * @private
		 */
		private function destroyController():void {
			if (_controller) _controller = null;
		}
		
		//---------------------------------------------------------------------
		//  Camera
		//---------------------------------------------------------------------
		
		/**
		 * @private
		 */
		private function setupCamera():void {
			_camera = new Camera();
			_camera.addController(_controller);
			
			_scene.addChild(_camera);
		}
		
		/**
		 * @private
		 */
		private function destroyCamera():void {
			if (_camera) {
				_scene.removeChild(_camera);
				_camera = null;
			}
		}
		
		
		/**
		 * Setup template.
		 */
		override public function setup():void {
			super.setup();
			
			setupViewport();
			setupScene();
			setupController();
			setupCamera();
			
			this.resize();
		}
		
		/**
		 * Destroy template.
		 */
		override public function destroy():void {
			destroyCamera();
			destroyController();
			destroyScene();
			destroyViewport();
			
			super.destroy();
		}
		
		/**
		 * Render template.
		 */
		override public function render():void {
			if (_scene && _viewport) _scene.render(_viewport);
		}
		
		
		/**
		 * @private
		 */
		public function get viewport():Viewport {
			return _viewport;
		}
		
		/**
		 * @private
		 */
		public function get camera():Camera {
			return _camera;
		}
		
		/**
		 * @private
		 */
		public function get scene():Scene {
			return _scene;
		}
		
		/**
		 * @private
		 */
		public function get controller():ArcBallController {
			return _controller;
		}
	}
}