/*
 * Copyright (c) 2012 David MASSENOT - http://artcustomer.fr/
 * 
 * Permission is hereby granted to use, modify, and distribute this file 
 * in accordance with the terms of the license agreement accompanying it.
 */

package artcustomer.framework.extensions.starling.layers.tools.scene {
	import flash.utils.getQualifiedClassName;
	
	import artcustomer.framework.engines.layers.tools.scene.AbstractScene3D;
	import artcustomer.framework.context.root.StarlingRootClass;
	import artcustomer.framework.extensions.starling.errors.FrameworkStarlingError;
	
	import starling.core.Starling;
	import starling.display.Sprite;
	
	
	/**
	 * AbstractStarlingScene
	 * 
	 * @author David Massenot
	 */
	public class AbstractStarlingScene extends AbstractScene3D {
		private static const FULL_CLASS_NAME:String = 'artcustomer.framework.extensions.starling.layers.tools.scene::AbstractStarlingScene';
		
		private var _starling:Starling;
		private var _rootClass:StarlingRootClass;
		
		private var _container:Sprite;
		
		private var _allowSetRoot:Boolean;
		
		
		/**
		 * Constructor
		 */
		public function AbstractStarlingScene() {
			super();
			
			if (getQualifiedClassName(this) == FULL_CLASS_NAME) throw new FrameworkStarlingError(FrameworkStarlingError.E_ABSTRACTSTARLINGSCENE_CONSTRUCTOR);
			
			_allowSetRoot = true;
		}
		
		//---------------------------------------------------------------------
		//  Container
		//---------------------------------------------------------------------
		
		/**
		 * @private
		 */
		private function setupContainer():void {
			_container = new Sprite();
		}
		
		/**
		 * @private
		 */
		private function destroyContainer():void {
			if (_container) _rootClass.removeChild(_container, true);
		}
		
		/**
		 * @private
		 */
		private function addContainer():void {
			if (_container) _rootClass.addChild(_container);
		}
		
		
		/**
		 * Setup StarlingScene.
		 */
		override public function setup():void {
			super.setup();
			
			_starling = Starling.current;
			
			setupContainer();
		}
		
		/**
		 * Destroy StarlingScene.
		 */
		override public function destroy():void {
			destroyContainer();
			
			_starling = null;
			_rootClass = null;
			_allowSetRoot = false;
			
			super.destroy();
		}
		
		/**
		 * Called when Starling is started and scene is ready. Override it and use it as entry point !
		 */
		public function start():void {
			
		}
		
		/**
		 * Called when Starling is resized.
		 */
		public function resize():void {
			
		}
		
		
		/**
		 * @private
		 */
		public function set rootClass(value:StarlingRootClass):void {
			if (_allowSetRoot) {
				_rootClass = value;
				_allowSetRoot = false;
				
				addContainer();
			}
		}
		
		/**
		 * @private
		 */
		public function get starling():Starling {
			return _starling;
		}
		
		/**
		 * @private
		 */
		public function get rootClass():StarlingRootClass {
			return _rootClass;
		}
		
		/**
		 * @private
		 */
		public function get container():Sprite {
			return _container;
		}
	}
}